#####################################################################
## Markov Chain Monte Carlo
##


## Metropolis sampler for the biological oxygen demand problem
bod.mh <- function(x,y,X1,q.sd,
                   alpha=0.01,beta=0.01,
                   iters=1000,sub=20) {

  ## The log posterior
  logp <- function(X) {
    ## (Log) prior
    lprior <- dgamma(X[3],alpha,beta,log=T)
    ## (Log) likelihood
    f <- X[1]*(1-exp(-X[2]*x))
    llikhd <- sum(dnorm(y,f,1/sqrt(X[3]),log=T))
    ## (Log) posterior
    lprior+llikhd
  }

  ## Initialize X and Xs
  X <- X1
  Xs <- matrix(0,iters,3)
  colnames(Xs) <- c("a","b","tau")

  ## Compute log posterior at X
  logp.X <- logp(X)

  for(k1 in 1:iters) {
    for(k2 in 1:sub) {
      ## Get next proposal point
      Y <- rnorm(3,X,q.sd)

      ## Ensure all components positive
      if(all(Y > 0)) {
        ## Compute log posterior at Y
        logp.Y <- logp(Y)
        ## Metropolis Hastings rule
        if(logp.Y - logp.X > log(runif(1))) {
          ## Proposal accepted - Y replaces X
          X <- Y
          logp.X <- logp.Y
        }
      }
    }
    ## Only sample the chain every sub iters
    Xs[k1,] <- X
  }
  Xs
}


## Read the biological oxygen demand data set
d <- read.table("bod.data",header=T)
## First run
fit <- bod.mh(d$Day,d$BOD,c(2.2,0.2,100),c(0.1,0.1,20),iters=1000)


## Estimate the posterior marginal means, standard deviations and
## quantiles
apply(fit[-(1:100),],2,function(x)
      c(mn=mean(x),sd=sqrt(var(x)),quantile(x,c(0.025,0.5,0.975))))


## Plot the first run
plot(as.ts(fit),main="First Run")


## Second run
fit <- bod.mh(d$Day,d$BOD,c(2.5,0.2,150),c(0.1,0.02,70),iters=2000)
## Estimate the posterior marginal means, standard deviations and
## quantiles
apply(fit[-(1:100),],2,function(x)
      c(mn=mean(x),sd=sqrt(var(x)),quantile(x,c(0.025,0.5,0.975))))


## Plot the second run
plot(as.ts(fit),main="Second Run")


## Pairwise plots
pairs(fit,pch=".")


## 2D kernel density estimate for a,b
library(MASS)
dns <- kde2d(fit[,1],fit[,2],n=80)
contour(dns,xlab="a",ylab="b")




